###############################################################################
# $Id: gdaltest.py,v 1.16 2004/07/09 18:38:30 warmerda Exp $
# 
# Project:  GDAL/OGR Test Suite
# Purpose:  Python Library supporting GDAL/OGR Test Suite
# Author:   Frank Warmerdam <warmerdam@pobox.com>
# 
###############################################################################
# Copyright (c) 2003, Frank Warmerdam <warmerdam@pobox.com>
# 
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
###############################################################################
# 
# $Log: gdaltest.py,v $
# Revision 1.16  2004/07/09 18:38:30  warmerda
# Added more diagnostics when projection doesn't work.
#
# Revision 1.15  2004/02/04 21:12:09  warmerda
# make sure '.' is in path
#
# Revision 1.14  2003/07/18 04:51:45  warmerda
# added approx_equal
#
# Revision 1.13  2003/04/22 17:59:38  warmerda
# supress spurious copycreate error messages about projections
#
# Revision 1.12  2003/04/02 10:42:04  dron
# Use osr.SpatialReference().IsSame() instead of string comparing in testSetProjection().
#
# Revision 1.11  2003/03/31 12:57:39  dron
# testSetNoDataValue() added.
#
# Revision 1.10  2003/03/28 09:13:15  dron
# New tests: testSetGeoTransform, testSetProjection, testSetMetadata,
# testSetNoDataValue.
#
# Revision 1.9  2003/03/27 15:52:09  dron
# Error handling improved.
#
# Revision 1.8  2003/03/26 20:00:28  dron
# Removed run_tests2() functioan and HashTest class.
#
# Revision 1.7  2003/03/25 17:28:58  dron
# Implemented GDALTest class.
#
# Revision 1.6  2003/03/25 11:13:48  dron
# Added run_tests2() function to run tests with list of parameters.
#
# Revision 1.5  2003/03/17 19:49:26  warmerda
# improved exception handling
#
# Revision 1.4  2003/03/17 05:43:11  warmerda
# added run_all function
#
# Revision 1.3  2003/03/13 16:33:14  warmerda
# dont reraise exceptions by default
#
# Revision 1.2  2003/03/13 16:23:49  warmerda
# added hash test class
#
# Revision 1.1  2003/03/05 05:04:44  warmerda
# New
#
#

import sys
import os
import gdal, osr

cur_name = 'default'

success_counter = 0
failure_counter = 0
blow_counter = 0
skip_counter = 0

reason = None

def setup_run( name ):
    global success_counter, failure_counter, blow_counter, skip_counter
    global cur_name
    
    cur_name = name

def run_tests( test_list ):
    global success_counter, failure_counter, blow_counter, skip_counter
    global cur_name, reason
    
    for test_item in test_list:
        if test_item is None:
            continue

        try:
            (func, name) = test_item
	    sys.stdout.write( '  TEST: ' + func.__name__[4:] + ': ' + name + ' ... ' )
	except:
            func = test_item
            name = func.__name__
	    sys.stdout.write( '  TEST: ' + name + ' ... ' )

        sys.stdout.flush()
            
        reason = None
        try:
            result = func()
            print result
        except:
            result = 'blowup'
            print result
            
            import traceback
            traceback.print_exc()


        if reason is not None:
            print '    ' + reason

        if result == 'success':
            success_counter = success_counter + 1
        elif result == 'fail':
            failure_counter = failure_counter + 1
        elif result == 'skip':
            skip_counter = skip_counter + 1
        else:
            blow_counter = blow_counter + 1

def post_reason( msg ):
    global reason

    reason = msg

def summarize():
    global success_counter, failure_counter, blow_counter, skip_counter
    global cur_name
    
    print
    print 'Test Script: %s' % cur_name
    print 'Succeeded: %d' % success_counter
    print 'Failed:    %d (%d blew exceptions)' \
          % (failure_counter+blow_counter, blow_counter)
    print 'Skipped:   %d' % skip_counter
    print
    
###############################################################################

class GDALTest:
    def __init__(self, drivername, filename, band, chksum,
		 xoff = 0, yoff = 0, xsize = 0, ysize = 0, options = [] ):
	self.driver = None
	self.drivername = drivername
	self.filename = filename
	self.band = band
	self.chksum = chksum
	self.xoff = xoff
	self.yoff = yoff
	self.xsize = xsize
	self.ysize = ysize
	self.options = options

    def testDriver(self):
	if self.driver is None:
	    self.driver = gdal.GetDriverByName( self.drivername )
	    if self.driver is None:
		post_reason( self.drivername + ' driver not found!' )
		return 'fail'

	return 'success'

    def testOpen(self):
	if self.testDriver() == 'fail':
	    return 'skip'

        ds = gdal.Open( 'data/' + self.filename, gdal.GA_ReadOnly )

        if ds is None:
            post_reason( 'Failed to open dataset: ' + self.filename )
            return 'fail'

        if self.xsize == 0 and self.ysize == 0:
            self.xsize = ds.RasterXSize
            self.ysize = ds.RasterYSize

        oBand = ds.GetRasterBand( self.band )
        chksum = oBand.Checksum( self.xoff, self.yoff, self.xsize, self.ysize )

        if chksum == self.chksum:
            return 'success'
        else:
            post_reason( 'Checksum for %s is %d, but expected %d.' \
                         % (self.filename, chksum, self.chksum) )
            return 'fail'
	

    def testCreateCopy(self):
	if self.testDriver() == 'fail':
	    return 'skip'

	src_ds = gdal.Open( 'data/' + self.filename )
	minmax = src_ds.GetRasterBand(self.band).ComputeRasterMinMax()

	new_filename = 'tmp/' + self.filename + '.tst'

        gdal.PushErrorHandler( 'CPLQuietErrorHandler' )
	new_ds = self.driver.CreateCopy( new_filename, src_ds,
	                                 options = self.options )
        gdal.PopErrorHandler()
	if new_ds is None:
	    post_reason( 'Failed to create test file using CreateCopy method.'\
                         + '\n' + gdal.GetLastErrorMsg() )
	    return 'fail'

	bnd = new_ds.GetRasterBand(self.band)
	if bnd.Checksum() != self.chksum:
	    post_reason( 'Did not get expected checksum on still-open file.' )
	    return 'fail'
	if bnd.ComputeRasterMinMax() != minmax:
	    post_reason( 'Did not get expected min/max values on still-open file.' )
	    return 'fail'

	bnd = None
	new_ds = None

	# hopefully it's closed now!

	new_ds = gdal.Open( new_filename )
	if new_ds is None:
	    post_reason( 'Failed to open dataset: ' + new_filename )
	    return 'fail'

	bnd = new_ds.GetRasterBand(self.band)
	if bnd.Checksum() != self.chksum:
	    post_reason( 'Did not get expected checksum on reopened file.' )
	    return 'fail'
	if bnd.ComputeRasterMinMax() != minmax:
	    post_reason( 'Did not get expected min/max values on reopened file.' )
	    return 'fail'

	bnd = None
	new_ds = None
	src_ds = None

	self.driver.Delete( new_filename )

	return 'success'

    def testCreate(self):
	if self.testDriver() == 'fail':
	    return 'skip'

	src_ds = gdal.Open( 'data/' + self.filename )
	xsize = src_ds.RasterXSize
	ysize = src_ds.RasterYSize
	src_img = src_ds.GetRasterBand(self.band).ReadRaster(0,0,xsize,ysize)
	minmax = src_ds.GetRasterBand(self.band).ComputeRasterMinMax()

	new_filename = 'tmp/' + self.filename + '.tst'
	new_ds = self.driver.Create( new_filename, xsize, ysize, 3,
				     src_ds.GetRasterBand(self.band).DataType,
				     options = self.options )
	if new_ds is None:
	    post_reason( 'Failed to create test file using Create method.' )
	    return 'fail'
	
	src_ds = None

	try:
	    new_ds.GetRasterBand(1).WriteRaster( 0, 0, xsize, ysize, src_img )
	    new_ds.GetRasterBand(2).WriteRaster( 0, 0, xsize, ysize, src_img )
	    new_ds.GetRasterBand(3).WriteRaster( 0, 0, xsize, ysize, src_img )
	except:
	    post_reason( 'Failed to write raster bands to test file.' )
	    return 'fail'

	if new_ds.GetRasterBand(1).Checksum() != self.chksum \
	   or new_ds.GetRasterBand(2).Checksum() != self.chksum \
	   or new_ds.GetRasterBand(3).Checksum() != self.chksum:
	    post_reason( 'Did not get expected checksum on still-open file.' )
	    return 'fail'
	if new_ds.GetRasterBand(1).ComputeRasterMinMax() != minmax \
	    or new_ds.GetRasterBand(2).ComputeRasterMinMax() != minmax \
	    or new_ds.GetRasterBand(3).ComputeRasterMinMax() != minmax:
	    post_reason( 'Did not get expected min/max values on still-open file.' )
	    return 'fail'
	
	new_ds = None

	new_ds = gdal.Open( new_filename )
	if new_ds is None:
	    post_reason( 'Failed to open dataset: ' + new_filename )
	    return 'fail'

	if new_ds.GetRasterBand(1).Checksum() != self.chksum \
	   or new_ds.GetRasterBand(2).Checksum() != self.chksum \
	   or new_ds.GetRasterBand(3).Checksum() != self.chksum:
	    post_reason( 'Did not get expected checksum on reopened file.' )
	    return 'fail'
	if new_ds.GetRasterBand(1).ComputeRasterMinMax() != minmax \
	    or new_ds.GetRasterBand(2).ComputeRasterMinMax() != minmax \
	    or new_ds.GetRasterBand(3).ComputeRasterMinMax() != minmax:
	    post_reason( 'Did not get expected min/max values on reopened file.' )
	    return 'fail'
	
	new_ds = None
	
	self.driver.Delete( new_filename )

	return 'success'

    def testSetGeoTransform(self):
	if self.testDriver() == 'fail':
	    return 'skip'

	src_ds = gdal.Open( 'data/' + self.filename )
	xsize = src_ds.RasterXSize
	ysize = src_ds.RasterYSize

	new_filename = 'tmp/' + self.filename + '.tst'
	new_ds = self.driver.Create( new_filename, xsize, ysize, 1,
				     src_ds.GetRasterBand(self.band).DataType,
				     options = self.options  )
	if new_ds is None:
	    post_reason( 'Failed to create test file using Create method.' )
	    return 'fail'
	
	gt = (123.0, 1.18, 0.0, 456.0, 0.0, -1.18 )
	if new_ds.SetGeoTransform( gt ) is not gdal.CE_None:
	    post_reason( 'Failed to set geographic transformation.' )
	    return 'fail'

	src_ds = None
	new_ds = None

	new_ds = gdal.Open( new_filename )
	if new_ds is None:
	    post_reason( 'Failed to open dataset: ' + new_filename )
	    return 'fail'

	if gt != new_ds.GetGeoTransform():
	    post_reason( 'Did not get expected geotransform.' )
	    return 'fail'

	new_ds = None
	
	self.driver.Delete( new_filename )

	return 'success'

    def testSetProjection(self):
	if self.testDriver() == 'fail':
	    return 'skip'

	src_ds = gdal.Open( 'data/' + self.filename )
	xsize = src_ds.RasterXSize
	ysize = src_ds.RasterYSize

	new_filename = 'tmp/' + self.filename + '.tst'
	new_ds = self.driver.Create( new_filename, xsize, ysize, 1,
				     src_ds.GetRasterBand(self.band).DataType,
				     options = self.options  )
	if new_ds is None:
	    post_reason( 'Failed to create test file using Create method.' )
	    return 'fail'
	
	gt = (123.0, 1.18, 0.0, 456.0, 0.0, -1.18 )
	prj="""PROJCS["WGS 84 / UTM zone 35N",GEOGCS["WGS 84",DATUM["WGS_1984",SPHEROID["WGS 84",6378137,298.2572235630016,AUTHORITY["EPSG","7030"]],AUTHORITY["EPSG","6326"]],PRIMEM["Greenwich",0],UNIT["degree",0.0174532925199433],AUTHORITY["EPSG","4326"]],PROJECTION["Transverse_Mercator"],PARAMETER["latitude_of_origin",0],PARAMETER["central_meridian",27],PARAMETER["scale_factor",0.9996],PARAMETER["false_easting",500000],PARAMETER["false_northing",0],UNIT["metre",1,AUTHORITY["EPSG","9001"]],AUTHORITY["EPSG","32635"]]"""

	src_osr = osr.SpatialReference()
	src_osr.ImportFromWkt(prj)
	
	new_ds.SetGeoTransform( gt )
	if new_ds.SetProjection( prj ) is not gdal.CE_None:
	    post_reason( 'Failed to set geographic projection string.' )
	    return 'fail'

	src_ds = None
	new_ds = None

	new_ds = gdal.Open( new_filename )
	if new_ds is None:
	    post_reason( 'Failed to open dataset: ' + new_filename )
	    return 'fail'

	new_osr = osr.SpatialReference()
	new_osr.ImportFromWkt(new_ds.GetProjection())
	if not new_osr.IsSame(src_osr):
	    post_reason( 'Did not get expected projection reference.' )
            print 'Got: '
            print new_osr.ExportToPrettyWkt()
            print 'Expected:'
            print src_osr.ExportToPrettyWkt()
	    return 'fail'

	new_ds = None
	
	self.driver.Delete( new_filename )

	return 'success'

    def testSetMetadata(self):
	if self.testDriver() == 'fail':
	    return 'skip'

	src_ds = gdal.Open( 'data/' + self.filename )
	xsize = src_ds.RasterXSize
	ysize = src_ds.RasterYSize

	new_filename = 'tmp/' + self.filename + '.tst'
	new_ds = self.driver.Create( new_filename, xsize, ysize, 1,
				     src_ds.GetRasterBand(self.band).DataType,
				     options = self.options  )
	if new_ds is None:
	    post_reason( 'Failed to create test file using Create method.' )
	    return 'fail'
	
	dict = {}
	dict['TEST_KEY'] = 'TestValue'
	new_ds.SetMetadata( dict )
# FIXME
	#if new_ds.SetMetadata( dict ) is not gdal.CE_None:
	    #print new_ds.SetMetadata( dict )
	    #post_reason( 'Failed to set metadata item.' )
	    #return 'fail'

	src_ds = None
	new_ds = None

	new_ds = gdal.Open( new_filename )
	if new_ds is None:
	    post_reason( 'Failed to open dataset: ' + new_filename )
	    return 'fail'

	md_dict = new_ds.GetMetadata()
	if md_dict['TEST_KEY'] != 'TestValue':
	    post_reason( 'Did not get expected metadata item.' )
	    return 'fail'

	new_ds = None
	
	self.driver.Delete( new_filename )

	return 'success'

    def testSetNoDataValue(self):
	if self.testDriver() == 'fail':
	    return 'skip'

	src_ds = gdal.Open( 'data/' + self.filename )
	xsize = src_ds.RasterXSize
	ysize = src_ds.RasterYSize

	new_filename = 'tmp/' + self.filename + '.tst'
	new_ds = self.driver.Create( new_filename, xsize, ysize, 1,
				     src_ds.GetRasterBand(self.band).DataType,
				     options = self.options  )
	if new_ds is None:
	    post_reason( 'Failed to create test file using Create method.' )
	    return 'fail'
	
	nodata = 11
	if new_ds.GetRasterBand(1).SetNoDataValue(nodata) is not gdal.CE_None:
	    post_reason( 'Failed to set NoData value.' )
	    return 'fail'

	src_ds = None
	new_ds = None

	new_ds = gdal.Open( new_filename )
	if new_ds is None:
	    post_reason( 'Failed to open dataset: ' + new_filename )
	    return 'fail'

	if nodata != new_ds.GetRasterBand(1).GetNoDataValue():
	    post_reason( 'Did not get expected NoData value.' )
	    return 'fail'

	new_ds = None
	
	self.driver.Delete( new_filename )

	return 'success'

    def testSetDescription(self):
	if self.testDriver() == 'fail':
	    return 'skip'

	src_ds = gdal.Open( 'data/' + self.filename )
	xsize = src_ds.RasterXSize
	ysize = src_ds.RasterYSize

	new_filename = 'tmp/' + self.filename + '.tst'
	new_ds = self.driver.Create( new_filename, xsize, ysize, 1,
				     src_ds.GetRasterBand(self.band).DataType,
				     options = self.options  )
	if new_ds is None:
	    post_reason( 'Failed to create test file using Create method.' )
	    return 'fail'
	
	description = "Description test string"
	new_ds.GetRasterBand(1).SetDescription(description)

	src_ds = None
	new_ds = None

	new_ds = gdal.Open( new_filename )
	if new_ds is None:
	    post_reason( 'Failed to open dataset: ' + new_filename )
	    return 'fail'

	if description != new_ds.GetRasterBand(1).GetDescription():
	    post_reason( 'Did not get expected description string.' )
	    return 'fail'

	new_ds = None
	
	self.driver.Delete( new_filename )

	return 'success'


###############################################################################

def run_all( dirlist, option_list ):

    for dir_name in dirlist:
        files = os.listdir(dir_name)

        old_path = sys.path
        sys.path.append('.')
        
        for file in files:
            if not file[-3:] == '.py':
                continue

            module = file[:-3]
            try:
                wd = os.getcwd()
                os.chdir( dir_name )
                
                exec "import " + module
                try:
                    exec "test_list = " + module + ".gdaltest_list"

                    print 'Running tests from %s/%s' % (dir_name,file)
                    run_tests( test_list )
                except:
                    pass
                
                os.chdir( wd )

            except:
                os.chdir( wd )
                print '... failed to load %s ... skipping.' % file

                import traceback
                traceback.print_exc()


        # We only add the tool directory to the python path long enough
        # to load the tool files.
        sys.path = old_path

def approx_equal( a, b ):
    a = float(a)
    b = float(b)
    if a == 0 and b != 0:
        return 0

    if abs(b/a - 1.0) > .00000000001:
        return 0
    else:
        return 1
    
